package main

import (
	"fmt"
	"log"

	"git.freifunk-franken.de/jkimmel/abbel/packet"
	"git.freifunk-franken.de/jkimmel/abbel/tlv"
)

type options struct {
	Group string
	Port  uint16
	Ifs   []string
}

func parseOpts() (options, error) {
	var opt options
	var err error

	opt = options{
		Group: "ff02:0:0:0:0:0:1:6",
		Port:  6696,
		Ifs:   []string{"babel"},
	}

	return opt, err
}

func run(opt options) error {
	conn, err := packet.ListenPort(opt.Port)
	if err != nil {
		return err
	}
	defer conn.Close()
	for _, iface := range opt.Ifs {
		if err = conn.JoinGroup(iface, opt.Group); err != nil {
			return err
		}
	}

	buf := [4096]byte{}
	var s tlv.PacketDecoder
	for {
		b, src, ifindex, err := conn.ReadFrom(buf[:])
		if err != nil {
			fmt.Println("Skipping packet:", err)
			continue
		}

		s.Reset(b, src, ifindex)
		for s.Scan() {
			switch t := s.TLV().(type) {
			case tlv.NextHop:
				fmt.Printf("%12s %s\n", t.T(), t.Address)
			case tlv.Hello:
				fmt.Printf("% 12s %+v\n", t.T(), t)
			case tlv.Update:
				if err != nil {
					fmt.Println(t, t.Prefix, err)
				} else {
					if t.Metric == 0xFFFF {
						fmt.Print("\x1B[31m")
					} else {
						fmt.Print("\x1B[36m")
					}
					fmt.Printf("% 12s %-43s%s\x1B[0m\n", t.T(), t.Prefix, t.FormatHeader())
				}
			case tlv.RouterID:
				fmt.Printf("%12s %s\n", t.T(), t)
			case tlv.RouteRequest:
				fmt.Printf("%12s %s\n", t.T(), t.Prefix)
			case tlv.Raw:
				fmt.Printf("   T: %12s (%2d), L: %3d\n", t.T(), t.T(), len(t.V()))
			case tlv.IHU:
				fmt.Printf("%12s %-43sRxcost %5d Interval %4d \n", t.T(), t.Address, t.Rxcost, t.Interval)
			default:
				if t != nil {
					fmt.Println("Unknown TLV", t.T(), t)
				} else {
					fmt.Println("got nil TLV")
				}
			}
			subtlv := s.SubTLV()
			for len(subtlv) > 0 {
				switch tlv.SubType(subtlv[0]) {
				case tlv.SubTypeSourcePrefix:
					var pfx tlv.SourcePrefix
					pfx, subtlv, err = tlv.SourcePrefixFromBytes(subtlv[2:])
					fmt.Print("\x1B[7;36m")
					fmt.Printf("%12s %s\x1B[0m\n", pfx.T(), pfx)
				case tlv.SubTypeDiversity, tlv.SubTypePad1, tlv.SubTypePadN, tlv.SubTypeTimestamp:
					fmt.Print("\x1B[7m")
					fmt.Printf("%12s: %v\x1B[0m\n", "Subtlv", subtlv)
					subtlv = nil
				default:
					panic("unknown subtlv type")
				}
			}
		}
		if s.Err() != nil {
			fmt.Println(err)
		}
	}
}

func main() {
	opt, err := parseOpts()
	if err != nil {
		log.Fatal(err)
	}
	if err := run(opt); err != nil {
		log.Fatal(err)
	}
}
